use std::collections::HashMap;

use gauntlet_common::model::Icons;
use gauntlet_common::model::ImageLike;
use gauntlet_common::model::UiWidgetId;
use iced::advanced::image::Handle;
use iced::widget::horizontal_space;
use iced::widget::image;
use iced::widget::value;
use iced_fonts::Bootstrap;
use iced_fonts::BOOTSTRAP_FONT;

use crate::ui::theme::text::TextStyle;
use crate::ui::theme::Element;
use crate::ui::theme::ThemableWidget;

pub fn render_image<'a, T: 'a + Clone>(
    images: &HashMap<UiWidgetId, Vec<u8>>,
    widget_id: UiWidgetId,
    image_data: &ImageLike,
    icon_style: Option<TextStyle>,
) -> Element<'a, T> {
    match image_data {
        ImageLike::ImageSource(_) => {
            match images.get(&widget_id) {
                Some(bytes) => image(Handle::from_bytes(bytes.clone())).into(),
                None => horizontal_space().into(),
            }
        }
        ImageLike::Icons(icon) => {
            match icon_style {
                None => value(icon_to_bootstrap(icon)).font(BOOTSTRAP_FONT).into(),
                Some(icon_style) => value(icon_to_bootstrap(icon)).font(BOOTSTRAP_FONT).themed(icon_style),
            }
        }
    }
}

pub fn icon_to_bootstrap(icon: &Icons) -> Bootstrap {
    match icon {
        Icons::Airplane => Bootstrap::Airplane,
        Icons::Alarm => Bootstrap::Alarm,
        Icons::AlignCentre => Bootstrap::AlignCenter,
        Icons::AlignLeft => Bootstrap::AlignStart,
        Icons::AlignRight => Bootstrap::AlignEnd,
        // Icons::Anchor => Bootstrap::,
        Icons::ArrowClockwise => Bootstrap::ArrowClockwise,
        Icons::ArrowCounterClockwise => Bootstrap::ArrowCounterclockwise,
        Icons::ArrowDown => Bootstrap::ArrowDown,
        Icons::ArrowLeft => Bootstrap::ArrowLeft,
        Icons::ArrowRight => Bootstrap::ArrowRight,
        Icons::ArrowUp => Bootstrap::ArrowUp,
        Icons::ArrowLeftRight => Bootstrap::ArrowLeftRight,
        Icons::ArrowsContract => Bootstrap::ArrowsAngleContract,
        Icons::ArrowsExpand => Bootstrap::ArrowsAngleExpand,
        Icons::AtSymbol => Bootstrap::At,
        // Icons::BandAid => Bootstrap::Bandaid,
        Icons::Cash => Bootstrap::Cash,
        // Icons::BarChart => Bootstrap::BarChart,
        // Icons::BarCode => Bootstrap::,
        Icons::Battery => Bootstrap::Battery,
        Icons::BatteryCharging => Bootstrap::BatteryCharging,
        // Icons::BatteryDisabled => Bootstrap::,
        Icons::Bell => Bootstrap::Bell,
        Icons::BellDisabled => Bootstrap::BellSlash,
        // Icons::Bike => Bootstrap::Bicycle,
        // Icons::Binoculars => Bootstrap::Binoculars,
        // Icons::Bird => Bootstrap::,
        Icons::Bluetooth => Bootstrap::Bluetooth,
        // Icons::Boat => Bootstrap::,
        Icons::Bold => Bootstrap::TypeBold,
        // Icons::Bolt => Bootstrap::,
        // Icons::BoltDisabled => Bootstrap::,
        Icons::Book => Bootstrap::Book,
        Icons::Bookmark => Bootstrap::Bookmark,
        Icons::Box => Bootstrap::Box,
        // Icons::Brush => Bootstrap::Brush,
        Icons::Bug => Bootstrap::Bug,
        Icons::Building => Bootstrap::Building,
        Icons::BulletPoints => Bootstrap::ListUl,
        Icons::Calculator => Bootstrap::Calculator,
        Icons::Calendar => Bootstrap::Calendar,
        Icons::Camera => Bootstrap::Camera,
        Icons::Car => Bootstrap::CarFront,
        Icons::Cart => Bootstrap::Cart,
        // Icons::Cd => Bootstrap::,
        // Icons::Center => Bootstrap::,
        Icons::Checkmark => Bootstrap::Checktwo,
        // Icons::ChessPiece => Bootstrap::,
        Icons::ChevronDown => Bootstrap::ChevronDown,
        Icons::ChevronLeft => Bootstrap::ChevronLeft,
        Icons::ChevronRight => Bootstrap::ChevronRight,
        Icons::ChevronUp => Bootstrap::ChevronUp,
        Icons::ChevronExpand => Bootstrap::ChevronExpand,
        Icons::Circle => Bootstrap::Circle,
        // Icons::CircleProgress100 => Bootstrap::,
        // Icons::CircleProgress25 => Bootstrap::,
        // Icons::CircleProgress50 => Bootstrap::,
        // Icons::CircleProgress75 => Bootstrap::,
        // Icons::ClearFormatting => Bootstrap::,
        Icons::Clipboard => Bootstrap::Clipboard,
        Icons::Clock => Bootstrap::Clock,
        Icons::Cloud => Bootstrap::Cloud,
        Icons::CloudLightning => Bootstrap::CloudLightning,
        Icons::CloudRain => Bootstrap::CloudRain,
        Icons::CloudSnow => Bootstrap::CloudSnow,
        Icons::CloudSun => Bootstrap::CloudSun,
        Icons::Code => Bootstrap::Code,
        Icons::Gear => Bootstrap::Gear,
        Icons::Coin => Bootstrap::Coin,
        Icons::Command => Bootstrap::Command,
        Icons::Compass => Bootstrap::Compass,
        // Icons::ComputerChip => Bootstrap::,
        // Icons::Contrast => Bootstrap::,
        Icons::CreditCard => Bootstrap::CreditCard,
        Icons::Crop => Bootstrap::Crop,
        // Icons::Crown => Bootstrap::,
        Icons::Document => Bootstrap::FileEarmark,
        Icons::DocumentAdd => Bootstrap::FileEarmarkPlus,
        Icons::DocumentDelete => Bootstrap::FileEarmarkX,
        Icons::Dot => Bootstrap::Dot,
        Icons::Download => Bootstrap::Download,
        // Icons::Duplicate => Bootstrap::,
        Icons::Eject => Bootstrap::Eject,
        Icons::ThreeDots => Bootstrap::ThreeDots,
        Icons::Envelope => Bootstrap::Envelope,
        Icons::Eraser => Bootstrap::Eraser,
        Icons::ExclamationMark => Bootstrap::ExclamationLg,
        Icons::Eye => Bootstrap::Eye,
        Icons::EyeDisabled => Bootstrap::EyeSlash,
        Icons::EyeDropper => Bootstrap::Eyedropper,
        Icons::Female => Bootstrap::GenderFemale,
        Icons::Film => Bootstrap::Film,
        Icons::Filter => Bootstrap::Filter,
        Icons::Fingerprint => Bootstrap::Fingerprint,
        Icons::Flag => Bootstrap::Flag,
        Icons::Folder => Bootstrap::Folder,
        Icons::FolderAdd => Bootstrap::FolderPlus,
        Icons::FolderDelete => Bootstrap::FolderMinus,
        Icons::Forward => Bootstrap::Forward,
        Icons::GameController => Bootstrap::Controller,
        Icons::Virus => Bootstrap::Virus,
        Icons::Gift => Bootstrap::Gift,
        Icons::Glasses => Bootstrap::Eyeglasses,
        Icons::Globe => Bootstrap::Globe,
        Icons::Hammer => Bootstrap::Hammer,
        Icons::HardDrive => Bootstrap::DeviceHdd,
        Icons::Headphones => Bootstrap::Headphones,
        Icons::Heart => Bootstrap::Heart,
        // Icons::HeartDisabled => Bootstrap::,
        Icons::Heartbeat => Bootstrap::Activity,
        Icons::Hourglass => Bootstrap::Hourglass,
        Icons::House => Bootstrap::House,
        Icons::Image => Bootstrap::Image,
        Icons::Info => Bootstrap::InfoLg,
        Icons::Italics => Bootstrap::TypeItalic,
        Icons::Key => Bootstrap::Key,
        Icons::Keyboard => Bootstrap::Keyboard,
        Icons::Layers => Bootstrap::Layers,
        // Icons::Leaf => Bootstrap::,
        Icons::LightBulb => Bootstrap::Lightbulb,
        Icons::LightBulbDisabled => Bootstrap::LightbulbOff,
        Icons::Link => Bootstrap::LinkFourfivedeg,
        Icons::List => Bootstrap::List,
        Icons::Lock => Bootstrap::Lock,
        // Icons::LockDisabled => Bootstrap::,
        Icons::LockUnlocked => Bootstrap::Unlock,
        // Icons::Logout => Bootstrap::,
        // Icons::Lowercase => Bootstrap::,
        // Icons::MagnifyingGlass => Bootstrap::,
        Icons::Male => Bootstrap::GenderMale,
        Icons::Map => Bootstrap::Map,
        Icons::Maximize => Bootstrap::Fullscreen,
        Icons::Megaphone => Bootstrap::Megaphone,
        Icons::MemoryModule => Bootstrap::Memory,
        Icons::MemoryStick => Bootstrap::UsbDrive,
        Icons::Message => Bootstrap::Chat,
        Icons::Microphone => Bootstrap::Mic,
        Icons::MicrophoneDisabled => Bootstrap::MicMute,
        Icons::Minimize => Bootstrap::FullscreenExit,
        Icons::Minus => Bootstrap::Dash,
        Icons::Mobile => Bootstrap::Phone,
        // Icons::Monitor => Bootstrap::,
        Icons::Moon => Bootstrap::Moon,
        // Icons::Mountain => Bootstrap::,
        Icons::Mouse => Bootstrap::Mouse,
        Icons::Multiply => Bootstrap::X,
        Icons::Music => Bootstrap::MusicNoteBeamed,
        Icons::Network => Bootstrap::BroadcastPin,
        Icons::Paperclip => Bootstrap::Paperclip,
        Icons::Paragraph => Bootstrap::TextParagraph,
        Icons::Pause => Bootstrap::Pause,
        Icons::Pencil => Bootstrap::Pencil,
        Icons::Person => Bootstrap::Person,
        Icons::PersonAdd => Bootstrap::PersonAdd,
        Icons::PersonRemove => Bootstrap::PersonDash,
        Icons::Phone => Bootstrap::Telephone,
        // Icons::PhoneRinging => Bootstrap::,
        Icons::PieChart => Bootstrap::PieChart,
        Icons::Capsule => Bootstrap::Capsule,
        // Icons::Pin => Bootstrap::,
        // Icons::PinDisabled => Bootstrap::,
        Icons::Play => Bootstrap::Play,
        Icons::Plug => Bootstrap::Plug,
        Icons::Plus => Bootstrap::Plus,
        // Icons::PlusMinusDivideMultiply => Bootstrap::,
        Icons::Power => Bootstrap::Power,
        Icons::Printer => Bootstrap::Printer,
        Icons::QuestionMark => Bootstrap::QuestionLg,
        Icons::Quotes => Bootstrap::Quote,
        Icons::Receipt => Bootstrap::Receipt,
        Icons::Repeat => Bootstrap::Repeat,
        Icons::Reply => Bootstrap::Reply,
        Icons::Rewind => Bootstrap::Rewind,
        Icons::Rocket => Bootstrap::Rocket,
        // Icons::Ruler => Bootstrap::,
        Icons::Shield => Bootstrap::Shield,
        Icons::Shuffle => Bootstrap::Shuffle,
        Icons::Snippets => Bootstrap::BodyText,
        Icons::Snowflake => Bootstrap::Snow,
        // Icons::VolumeHigh => Bootstrap::VolumeUp,
        // Icons::VolumeLow => Bootstrap::VolumeDown,
        // Icons::VolumeOff => Bootstrap::VolumeOff,
        // Icons::VolumeOn => Bootstrap::,
        Icons::Star => Bootstrap::Star,
        // Icons::StarDisabled => Bootstrap::,
        Icons::Stop => Bootstrap::Stop,
        Icons::Stopwatch => Bootstrap::Stopwatch,
        Icons::StrikeThrough => Bootstrap::TypeStrikethrough,
        Icons::Sun => Bootstrap::SunFill, // TODO why Sun isn't in iced_aw?
        Icons::Scissors => Bootstrap::Scissors,
        // Icons::Syringe => Bootstrap::,
        Icons::Tag => Bootstrap::Tag,
        Icons::Thermometer => Bootstrap::Thermometer,
        Icons::Terminal => Bootstrap::Terminal,
        Icons::Text => Bootstrap::Fonts,
        Icons::TextCursor => Bootstrap::CursorText,
        // Icons::TextSelection => Bootstrap::,
        // Icons::Torch => Bootstrap::,
        // Icons::Train => Bootstrap::,
        Icons::Trash => Bootstrap::Trash,
        Icons::Tree => Bootstrap::Tree,
        Icons::Trophy => Bootstrap::Trophy,
        Icons::People => Bootstrap::People,
        Icons::Umbrella => Bootstrap::Umbrella,
        Icons::Underline => Bootstrap::TypeUnderline,
        Icons::Upload => Bootstrap::Upload,
        // Icons::Uppercase => Bootstrap::,
        Icons::Wallet => Bootstrap::Wallet,
        Icons::Wand => Bootstrap::Magic,
        // Icons::Warning => Bootstrap::,
        // Icons::Weights => Bootstrap::,
        Icons::Wifi => Bootstrap::Wifi,
        Icons::WifiDisabled => Bootstrap::WifiOff,
        Icons::Window => Bootstrap::Window,
        Icons::Tools => Bootstrap::Tools,
        Icons::Watch => Bootstrap::Watch,
        Icons::XMark => Bootstrap::XLg,
        Icons::Indent => Bootstrap::Indent,
        Icons::Unindent => Bootstrap::Unindent,
    }
}
